<?php
namespace Controleur;

use Gateway\Connection;
use Gateway\UserGateway;
use Model\UserModel;
use Twig\Error\LoaderError;
use Twig\Error\RuntimeError;
use Twig\Error\SyntaxError;

Class VisitorControler {

    private Connection $co;
    private UserGateway $gw;

    private UserModel $mdl;

    private array $errors;

    /**
     * @throws SyntaxError
     * @throws RuntimeError
     * @throws LoaderError
     */
    public function __construct(Connection $co, string $action) {
        $this->co = $co;
        $this->gw = new UserGateway($this->co);
        $this->mdl = new UserModel($this->gw);
        $this -> getAction($action);
    }

    /**
     * @throws SyntaxError
     * @throws RuntimeError
     * @throws LoaderError
     */
    public function getAction(string $action): void
    {
        switch ($action) {
            case "login":
                $this -> visitorLogIn();
                break;
            case "signin":
                $this -> visitorSignIn();
                break;
            case "valid":
                $this -> toSignIn();
                break;
            default:
                break;
        }
    }

    public function accueil(){
        global $vues;
        require_once $vues['accueil'];
    }

    public function quote(array $arg){
        global $vues;
        $id=$arg['idQuote'] ?? 1;

        //echo "{$id}";
        require_once $vues['quote'];
    }

    /**
     * @throws SyntaxError
     * @throws RuntimeError
     * @throws LoaderError
     */
    public function visitorLogIn(): void
    {
        global $twig;
        echo $twig->render("login.html.twig");

        $this -> toLogIn();
    }

    public function toLogIn() : void
    {
        if ($_POST)
        {
            $pseudo = $_POST['pseudo'] ?? null;
            $mdp = $_POST['mdp'] ?? null;

            $user = $this -> mdl -> getUsername($pseudo);

            if ($user)
            {
                if (password_verify($mdp, $user->getPassword()))
                {
                    $_SESSION['user'] = $pseudo;
                    header("Location: /");
                }
                else die(); /* Afficher Erreur Mot de passe */
            }
            else die(); /* Afficher Utilisateur non trouvé */
        }

    }

    /**
     * @throws SyntaxError
     * @throws RuntimeError
     * @throws LoaderError
     */
    public function visitorSignIn(): void
    {
        global $twig;
        echo $twig->render("signin.html.twig");
    }


    /**
     * @throws RuntimeError
     * @throws SyntaxError
     * @throws LoaderError
     */
    public function toSignIn() : void
    {
        global $twig;

        $this -> errors = [null, null, null];

        if ($_POST) {
            $pseudo = $_POST['pseudo'] ?? null;
            $email = $_POST['email'] ?? null;
            $mdp = $_POST['mdp'] ?? null;
            $cmdp = $_POST['cmdp'] ?? null;

            if ($mdp != $cmdp) {
                $this->errors[2] = "Mots de passe incorrects";
                //$this->visitorSignIn();
                echo $twig->render("signin.html.twig", ['error' => $this->errors]);
                exit();
            }
            $option = ['cost' => 12];
            $hmdp = password_hash($mdp, PASSWORD_BCRYPT, $option);

            $isUserAlreadyUsed = $this -> mdl -> getUsername($pseudo);
            $isEmailAlreadyUsed = $this -> mdl -> getEmail($email);

            if ($isUserAlreadyUsed)  {
                $this->errors[0] = "Pseudo déjà utilisé";
                echo $twig->render("signin.html.twig", ['error' => $this->errors]);
                exit();
            }
            else if ($isEmailAlreadyUsed)  {
                $this->errors[1] = "Email déjà utilisé";
                echo $twig->render("signin.html.twig", ['error' => $this->errors]);
                exit();
            }
            else echo $this->mdl->insertUser($pseudo, $email, $hmdp);

            $_SESSION["pseudo"] = $pseudo;

            header("Location: /");
        }
    }
}


